@@ -22,13 +22,12 @@
 #define mb() do {  asm volatile("bcr 15,0" : : : "memory"); } while (0)
 #endif
 
-static inline void osb(void)
-{
-	asm volatile(
-		ALTERNATIVE("", ".long 0xb2e8f000", 81)
-		: : : "memory");
-}
-#define osb osb
+/* Prevent speculative execution past this barrier. */
+#define barrier_nospec()						\
+	do {								\
+		asm volatile(ALTERNATIVE("", ".long 0xb2e8f000", 81)	\
+			     : : : "memory");				\
+	} while (0)
 
 #define rmb()				mb()
 #define wmb()				mb()
