@@ -22,6 +22,7 @@
 #include <linux/ctype.h>
 #include <linux/projid.h>
 #include <linux/fs_struct.h>
+#include <linux/nospec.h>
 
 /*
  * sysctl determining whether unprivileged users may unshare a new
@@ -500,7 +501,7 @@ static void *m_start(struct seq_file *seq, loff_t *ppos, struct uid_gid_map *map
 	loff_t pos = *ppos;
 
 	if (pos < map->nr_extents) {
-		osb();
+		pos = array_index_nospec((unsigned long)pos, map->nr_extents); /* needed? */
 		extent = &map->extent[pos];
 	}
 
